@@ -478,7 +478,7 @@ func TestIntegrationChannelIDsExhausted(t *testing.T) {
 	}
 	defer c.Close()
 
-	for i := 1; i <= c.Config.ChannelMax; i++ {
+	for i := uint16(1); i <= c.Config.ChannelMax; i++ {
 		if _, err := c.Channel(); err != nil {
 			t.Fatalf("expected allocating all channel ids to succed, failed on %d with %v", i, err)
 		}
@@ -2025,6 +2025,59 @@ func TestIntegrationGetNextPublishSeqNo(t *testing.T) {
 	}
 }
 
+func TestIntegrationGetNextPublishSeqNoRace(t *testing.T) {
+	if c := integrationConnection(t, "GetNextPublishSeqNoRace"); c != nil {
+		defer c.Close()
+
+		ch, err := c.Channel()
+		if err != nil {
+			t.Fatalf("channel: %v", err)
+		}
+
+		if err = ch.Confirm(false); err != nil {
+			t.Fatalf("could not confirm")
+		}
+
+		ex := "test-get-next-pub"
+		if err = ch.ExchangeDeclare(ex, "direct", false, false, false, false, nil); err != nil {
+			t.Fatalf("cannot declare %v: got: %v", ex, err)
+		}
+
+		n := ch.GetNextPublishSeqNo()
+		if n != 1 {
+			t.Fatalf("wrong next publish seqence number before any publish, expected: %d, got: %d", 1, n)
+		}
+
+		wg := sync.WaitGroup{}
+		fail := false
+
+		wg.Add(2)
+
+		go func() {
+			defer wg.Done()
+			_ = ch.GetNextPublishSeqNo()
+		}()
+
+		go func() {
+			defer wg.Done()
+			if err := ch.PublishWithContext(context.TODO(), "test-get-next-pub-seq", "", false, false, Publishing{}); err != nil {
+				t.Logf("publish error: %v", err)
+				fail = true
+			}
+		}()
+
+		wg.Wait()
+		if fail {
+			t.FailNow()
+		}
+
+		n = ch.GetNextPublishSeqNo()
+		if n != 2 {
+			t.Fatalf("wrong next publish seqence number after 15 publishing, expected: %d, got: %d", 2, n)
+		}
+	}
+}
+
 // https://github.com/rabbitmq/amqp091-go/pull/44
 func TestShouldNotWaitAfterConnectionClosedIssue44(t *testing.T) {
 	conn := integrationConnection(t, "TestShouldNotWaitAfterConnectionClosedIssue44")
